--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 20.12.2020

About:
Idea inspired by vehicle sales feature available in previous versions of FS/LS

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


StoreSale = {}

StoreSale.MIN_START_HOUR = 7
StoreSale.MAX_START_HOUR = 17

StoreSale.MAX_STORE_ITEM_CHECKS = 10

StoreSale.NOTIFICATION_DURATION = 20000
StoreSale.NOTIFICATION_COLOUR = {
    1,
    1,
    1,
    1
}

StoreSale.INVALID_CATEGORY = {
    PALLETS = "PALLETS",
    BIGBAGS = "BIGBAGS",
    BALES = "BALES"
}

local StoreSale_mt = Class(StoreSale)


function StoreSale:new(isServer, isClient, customEnvironment, isBrandSale, customMt)
    local self = setmetatable({}, customMt or StoreSale_mt)

    self.isServer = isServer
    self.isClient = isClient

    self.customEnvironment = customEnvironment
    self.isBrandSale = Utils.getNoNil(isBrandSale, false)

    self:initDataStructures()

    return self
end

function StoreSale:initDataStructures()
    self.xmlFilename = nil
    self.brandIndex = nil

    self.startDay = 0
    self.startHour = 0

    self.remainingHours = 0
    self.discountPercent = 0

    self.isValid = false
    self.isActive = false
end

function StoreSale:delete()
    self.isValid = false
end

function StoreSale:saveToXMLFile(xmlFile, key)
    if self:getIsValid() and self.xmlFilename ~= nil then
        setXMLString(xmlFile, key .. "#xmlFilename", HTMLUtil.encodeToHTML(NetworkUtil.convertToNetworkFilename(self.xmlFilename)))

        if self.lastBrandIndex ~= nil then
            local brand = g_brandManager:getBrandByIndex(self.lastBrandIndex)

            if brand ~= nil then
                setXMLString(xmlFile, key .. "#lastBrandName", brand.name)
            end
        end

        setXMLInt(xmlFile, key .. "#remainingHours", self.remainingHours or 0)
        setXMLInt(xmlFile, key .. "#discountPercent", self.discountPercent or 0)

        setXMLInt(xmlFile, key .. "#startDay", self.startDay or 0)
        setXMLInt(xmlFile, key .. "#startHour", self.startHour or 0)

        setXMLBool(xmlFile, key .. "#isActive", self.isActive or false)
    end
end

function StoreSale:loadFromXMLFile(xmlFile, key)
    local xmlFilename = getXMLString(xmlFile, key .. "#xmlFilename")

    if xmlFilename ~= nil then
        local item = g_storeManager:getItemByXMLFilename(NetworkUtil.convertFromNetworkFilename(xmlFilename))

        if item ~= nil then
            self.xmlFilename = item.xmlFilename
            self.brandIndex = StoreSale.getValidBrandIndex(item.xmlFilename)

            local brand = g_brandManager:getBrandByName(getXMLString(xmlFile, key .. "#lastBrandName"))

            if brand ~= nil then
                self.lastBrandIndex = brand.index
            end

            self.remainingHours = math.max(getXMLInt(xmlFile, key .. "#remainingHours") or 0, 0)
            self.discountPercent = math.max(getXMLInt(xmlFile, key .. "#discountPercent") or 0, 0)

            self.startDay = math.max(getXMLInt(xmlFile, key .. "#startDay") or 0, 0)
            self.startHour = math.max(getXMLInt(xmlFile, key .. "#startHour") or 0, 0)

            self.isValid = self.remainingHours > 0 and self.discountPercent > 0 and self.startDay > 0 and self.startHour > 0

            if self.isValid then
                self:setActive(getXMLBool(xmlFile, key .. "#isActive"))
            else
                local environment = g_currentMission.environment
                self:createSale(environment.currentDay, environment.currentHour)
            end
        end
    end
end

function StoreSale:updateSale(currentDay, currentHour)
    local saleUpdated = false

    if self.isServer then
        if self.isValid then
            if self.isActive then
                self.remainingHours = math.max((self.remainingHours or 0) - 1, 0)

                if self.remainingHours <= 0 then
                    self:setActive(false)
                    self:createSale(currentDay, currentHour) -- Old sale ended so try and create a new one
                end

                saleUpdated = true
            else
                if self.startDay == currentDay then
                    if self.startHour <= currentHour then
                        self:setActive(true)
                        saleUpdated = true
                    end
                elseif self.startDay < currentDay then
                    self:createSale(currentDay, currentHour) -- If the start day has passed something went wrong so try and create a new sale
                    saleUpdated = true
                end
            end
        else
            self:createSale(currentDay, currentHour) -- Try to create a valid sale
            saleUpdated = true
        end
    end

    return saleUpdated
end

function StoreSale:createSale(currentDay, currentHour)
    self:initDataStructures() -- Reset the sale and invalidate it

    local items = g_storeManager:getItems()
    local numStoreItems = #items

    for i = 1, StoreSale.MAX_STORE_ITEM_CHECKS do
        local item = items[math.random(numStoreItems)]

        if StoreSale.getIsValidStoreItem(item) and (not self.isBrandSale or item.brandIndex ~= self.lastBrandIndex) then
            self.xmlFilename = item.xmlFilename
            self.brandIndex = item.brandIndex

            if self.isBrandSale then
                self.startDay = currentDay + StoreSale.getRandomValue(5, 10) -- Brand sales start 5 - 10 days from current day
                self.lastBrandIndex = self.brandIndex
            else
                self.startDay = currentDay + StoreSale.getRandomValue(2, 6) -- Item sales start 2 - 6 days from current day
            end

            self.startHour = StoreSale.getRandomValue(StoreSale.MIN_START_HOUR, StoreSale.MAX_START_HOUR) -- Only start sales during store hours

            self.remainingHours = StoreSale.getRandomValue(2, 12, 4) -- Set duration from 8 - 48 hours
            self.discountPercent = StoreSale.getRandomValue(1, 4, 5) -- Set discount percent from 5 - 20 percent

            self.isValid = true

            return true
        end
    end

    return false
end

function StoreSale:setActive(isActive)
    isActive = self.isValid and Utils.getNoNil(isActive, false)

    if self.isActive ~= isActive then
        self.isActive = isActive

        if g_storeSalesManager ~= nil then
            if self.isBrandSale then
                g_storeSalesManager:updateBrandSaleObjects(self)
            end

            g_storeSalesManager:updateShopMenu()
        end

        if isActive and g_dedicatedServerInfo == nil then
            local notification = self:getSideNotification()

            if notification ~= "" and g_currentMission ~= nil then
                g_currentMission.hud:addSideNotification(StoreSale.NOTIFICATION_COLOUR, notification, StoreSale.NOTIFICATION_DURATION)
            end
        end
    end

    return isActive
end

function StoreSale:getIsActive()
    return self.isActive
end

function StoreSale:getIsValid()
    return self.isValid and self.remainingHours > 0
end

function StoreSale:getIsBrandSale()
    return self.isBrandSale
end

function StoreSale:getIsSaleItem(item)
    if self:getIsValid() and self.isActive then
        if self.isBrandSale then
            return StoreSale.getIsValidStoreItem(item) and item.brandIndex == self.brandIndex
        end

        return item.xmlFilename == self.xmlFilename
    end

    return false
end

function StoreSale:getSideNotification()
    if self.isBrandSale then
        local brand = g_brandManager:getBrandByIndex(self.brandIndex)

        if brand ~= nil then
            return string.format(g_i18n:getText("storeSales_brandSaleMessage", self.customEnvironment), brand.title)
        end
    else
        local item = g_storeManager:getItemByXMLFilename(self.xmlFilename)

        if item ~= nil then
            local name = item.name
            local brand = g_brandManager:getBrandByIndex(item.brandIndex)

            if brand ~= nil then
                name = brand.title .. " " .. name
            end

            return string.format(g_i18n:getText("storeSales_itemSaleMessage", self.customEnvironment), name)
        end
    end

    return ""
end

function StoreSale.getValidBrandIndex(xmlFilename)
    local item = g_storeManager:getItemByXMLFilename(xmlFilename)

    if item ~= nil then
        return item.brandIndex
    end

    return Brand.LIZARD
end

function StoreSale.getIsValidStoreItem(item)
    return StoreItemUtil.getIsVehicle(item) and StoreSale.getIsValidStoreCategory(item.categoryName) and item.showInStore and not item.isBundleItem
end

function StoreSale.getIsValidStoreCategory(categoryName)
    return StoreSale.INVALID_CATEGORY[categoryName] == nil
end

function StoreSale.getRandomValue(minValue, maxValue, multiplier)
    return math.random(minValue, maxValue) * (multiplier or 1)
end
