--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 20.12.2020

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


local loadError
local storeSalesManager

local dynamicallyLoadedBrandSaleNode
local dynamicallyLoadedBrandSaleMaps = {
    ["mapDE"] = "baseGame",
    ["mapUS"] = "baseGame",
    ["mapSA"] = "FS19_EstanciaLapacho",
    ["alpineFarmingPack"] = "pdlc_alpineFarmingPack"
}

local buildId = 2
local versionString = "0.0.0.0"

local modName = g_currentModName
local modDirectory = g_currentModDirectory


local function getIsLoaded()
    return storeSalesManager ~= nil
end

local function indexToNode(rootNode, indexString, validName)
    local node = rootNode or getRootNode()
    local childIndexs = StringUtil.splitString("|", indexString)

    for i = 1, #childIndexs do
        local index = tonumber(childIndexs[i])
        if getNumOfChildren(node) >= index then
            node = getChildAt(node, index)
        else
            break
        end
    end

    return node, getName(node) == (validName or "")
end

local function shopControllerLoad(shopController)
    -- Add new brand 'sales' and add to table position 1. Doing this late so it is always first in the list
    if not shopController.isSalesInitialized and getIsLoaded() then
        local label = g_i18n:getText("category_sales")
        local imageFilename = "store/brand_sales.dds"

        local brand = g_brandManager:addBrand("SALES", label, imageFilename, modDirectory, true, imageFilename)

        table.insert(shopController.displayBrands, 1, {
            label = label,
            id = brand.index,
            sortValue = "AAA_SALES",
            iconFilename = brand.imageShopOverview
        })

        shopController.isSalesInitialized = true
    end
end

local function economyManagerGetBuyPrice(economyManager, superFunc, storeItem, boughtConfigurations)
    -- In early versions of FS 'sale' would be returned when active, this is missing in 19 but still seems to populate the 'SALES' brand in Store UI so I just create my own sale here
    local price, upgradePrice, sale = superFunc(economyManager, storeItem, boughtConfigurations)

    if sale == nil and getIsLoaded() then
        sale = storeSalesManager:getIsSaleItem(storeItem)
    end

    if sale ~= nil then
        price = math.floor((price - upgradePrice) * (1 - sale.discountPercent * 0.01) + upgradePrice)
    end

    return price, upgradePrice, sale
end

local function shopItemsFrameGetStoreItemDisplayPrice(shopItemsFrame, superFunc, storeItem, realItem, isSelling)
    -- Add a visible discount to the store price so it is easy to determine a sale item
    local price = superFunc(shopItemsFrame, storeItem, realItem, isSelling)

    if not isSelling and getIsLoaded() then
        local sale = storeSalesManager:getIsSaleItem(storeItem)

        if sale ~= nil then
            price = string.format("%s  (-%d%%)", price, sale.discountPercent)
        end
    end

    return price
end

local function getSpecValueSaleRemaingHours(storeItem, realItem)
    -- Display the new spec type 'saleRemaingHours' if not displaying garage or leased pages to show the remaining sales time
    if getIsLoaded() and realItem == nil and StoreItemUtil.getIsVehicle(storeItem) and storeItem.showInStore and not storeItem.isBundleItem then
        local sale = storeSalesManager:getIsSaleItem(storeItem)

        if sale ~= nil then
            return string.format("%s %d h", g_i18n:getText("shop_saleRemaingHours"), sale.remainingHours)
        end
    end

    return nil
end

local function saveToXMLFile(missionInfo)
    if getIsLoaded() and missionInfo.isValid then
        local xmlFile = createXMLFile("StoreSalesXML", missionInfo.savegameDirectory .. "/storeSales.xml", "storeSales")

        if xmlFile ~= nil then
            setXMLInt(xmlFile, "storeSales#buildId", buildId)
            setXMLString(xmlFile, "storeSales#versionString", versionString)

            storeSalesManager:saveToXMLFile(xmlFile, "storeSales")

            saveXMLFile(xmlFile)
            delete(xmlFile)
        end
    end
end

local function loadMap(mission, mapFilename)
    if not getIsLoaded() then
        if g_storeManager.nameToSpecType["saleRemaingHours"] == nil then
            g_storeManager:addSpecType("saleRemaingHours", "shopListAttributeIconStoreSale", nil, getSpecValueSaleRemaingHours)
        end

        storeSalesManager = StoreSalesManager:new(mission:getIsServer(), mission:getIsClient(), modName, modDirectory)

        if storeSalesManager ~= nil then
            local filename = modDirectory .. "shared/dynamicallyLoadedBrandSales.i3d"

            if fileExists(filename) then
                local mapModName, _ = Utils.getModNameAndBaseDirectory(mapFilename)

                if mapModName == nil then
                    mapModName = "baseGame"
                end

                for name, env in pairs (dynamicallyLoadedBrandSaleMaps) do
                    if mapFilename:find(name) ~= nil and env == mapModName then
                        local i3dNode = loadI3DFile(filename, false, true, false)

                        if i3dNode ~= 0 then
                            for i = 0, getNumOfChildren(i3dNode) - 1 do
                                local child = getChildAt(i3dNode, i)

                                if getName(child) == name then
                                    unlink(child)
                                    dynamicallyLoadedBrandSaleNode = child

                                    break
                                end
                            end

                            delete(i3dNode)
                        end
                    end
                end
            end

            getfenv(0)["g_storeSalesManager"] = storeSalesManager
        end
    end
end

local function loadMapFinished(mission, node)
    if getIsLoaded() then
        if dynamicallyLoadedBrandSaleNode ~= nil then
            if node ~= 0 and BrandSaleObjects ~= nil then
                if storeSalesManager.brandSaleObjects == nil then
                    local hideOnSaleObjects, brandSaleObjectsValid = nil, true

                    if getName(dynamicallyLoadedBrandSaleNode) == "mapUS" then
                        local bannersNode, bannersValid = indexToNode(node, "17|0|10|5|9|8|11", "banners")

                        if bannersValid then
                            hideOnSaleObjects = {
                                node = bannersNode,
                                collisionMask = getCollisionMask(bannersNode),
                                setCollision = true
                            }
                        else
                            brandSaleObjectsValid = false
                        end
                    end

                    if brandSaleObjectsValid then
                        link(node, dynamicallyLoadedBrandSaleNode)

                        local brandSaleObjects = BrandSaleObjects:new(dynamicallyLoadedBrandSaleNode, storeSalesManager)

                        if brandSaleObjects ~= nil and brandSaleObjects.isValid then
                            if hideOnSaleObjects ~= nil then
                                brandSaleObjects.hideOnSaleObjects = {hideOnSaleObjects}
                            end

                            local decalSale2Node, decalSale2Valid = indexToNode(node, "17|2|10|11", "decalSale2")
                            local decalSale1Node, decalSale1Valid = indexToNode(node, "17|2|10|12", "decalSale1")

                            if decalSale2Valid then
                                setVisibility(decalSale2Node, false)

                                table.insert(brandSaleObjects.objects, {
                                    node = decalSale2Node,
                                    isSign = true,
                                    setCollision = false
                                })
                            end

                            if decalSale1Valid then
                                setVisibility(decalSale1Node, false)

                                table.insert(brandSaleObjects.objects, {
                                    node = decalSale1Node,
                                    isSign = true,
                                    setCollision = false
                                })
                            end

                            g_storeSalesManager.brandSaleObjects = brandSaleObjects
                        else
                            brandSaleObjectsValid = false
                        end
                    end

                    if not brandSaleObjectsValid then
                        delete(dynamicallyLoadedBrandSaleNode)
                        dynamicallyLoadedBrandSaleNode = nil
                    end
                end
            end
        end

        if mission:getIsServer() and mission.missionInfo.savegameDirectory ~= nil and fileExists(mission.missionInfo.savegameDirectory .. "/storeSales.xml") then
            local xmlFile = loadXMLFile("StoreSalesXML", mission.missionInfo.savegameDirectory .. "/storeSales.xml")

            if xmlFile ~= nil then
                storeSalesManager:loadFromXMLFile(xmlFile, "storeSales")
                delete(xmlFile)
            end
        end
    end
end

local function connectionFinishedLoading(mission, superFunc, connection, x, y, z, viewDistanceCoeff)
    if getIsLoaded() then
        local oldGreatDemandsEvent_new = GreatDemandsEvent.new

        function GreatDemandsEvent.new(...)
            connection:sendEvent(StoreSalesEvent:new(storeSalesManager:getSales()))

            return oldGreatDemandsEvent_new(...)
        end
    end

    return superFunc(mission, connection, x, y, z, viewDistanceCoeff)
end

local function unloadMap(mission)
    if getIsLoaded() then
        storeSalesManager:delete()

        storeSalesManager = nil
        getfenv(0)["g_storeSalesManager"] = nil
    end

    if dynamicallyLoadedBrandSaleNode ~= nil then
        delete(dynamicallyLoadedBrandSaleNode)
        dynamicallyLoadedBrandSaleNode = nil
    end
end

local function init()
    if g_storeSalesManager == nil then
        local mod = g_modManager:getModByName(modName)

        versionString = tostring(mod.version) or versionString

        if mod.modName == "FS19_StoreSales" or mod.modName == "FS19_StoreSales_update" and mod.author:len() == 3 then
            source(modDirectory .. "scripts/misc/BrandSaleObjects.lua")
            source(modDirectory .. "scripts/misc/StoreSalesRender.lua")

            source(modDirectory .. "scripts/events/StoreSalesEvent.lua")
            source(modDirectory .. "scripts/StoreSale.lua")

            source(modDirectory .. "scripts/StoreSalesManager.lua")

            -- Add onCreate function so map can use it on load, works in 19 so this is great :-)
            g_onCreateUtil.addOnCreateFunction("BrandSaleObjects", BrandSaleObjects.onCreate)

            -- Load new spec type icon
            local xmlFile = loadXMLFileFromMemory("profileXML", string.format([[
<?xml version="1.0" encoding="utf-8" standalone="no" ?>

<GUIProfiles>
    <Profile name="shopListAttributeIconStoreSale" extends="shopListAttributeIcon">
        <Value name="imageFilename" value="%sstore/iconSaleRemainingHours.dds"/>
        <Value name="imageUVs" value="0 0 1 1"/>
    </Profile>
</GUIProfiles>]], modDirectory))

            g_gui:loadProfileSet(xmlFile, "GuiProfiles", {})
            delete(xmlFile)

            ShopController.load = Utils.appendedFunction(ShopController.load, shopControllerLoad)
            EconomyManager.getBuyPrice = Utils.overwrittenFunction(EconomyManager.getBuyPrice, economyManagerGetBuyPrice)
            ShopItemsFrame.getStoreItemDisplayPrice = Utils.overwrittenFunction(ShopItemsFrame.getStoreItemDisplayPrice, shopItemsFrameGetStoreItemDisplayPrice)

            FSCareerMissionInfo.saveToXMLFile = Utils.appendedFunction(FSCareerMissionInfo.saveToXMLFile, saveToXMLFile)

            FSBaseMission.loadMap = Utils.prependedFunction(FSBaseMission.loadMap, loadMap)
            FSBaseMission.loadMapFinished = Utils.prependedFunction(FSBaseMission.loadMapFinished, loadMapFinished)
            FSBaseMission.onConnectionFinishedLoading = Utils.overwrittenFunction(FSBaseMission.onConnectionFinishedLoading, connectionFinishedLoading)
            FSBaseMission.delete = Utils.prependedFunction(FSBaseMission.delete, unloadMap)
        else
            loadError = {
                startUpdateTime = 2100,

                update = function(self, dt)
                    self.startUpdateTime = self.startUpdateTime - dt

                    if self.startUpdateTime < 0 then
                        local text = string.format(g_i18n:getText("storeSales_loadError", modName), modName, mod.author)

                        if g_dedicatedServerInfo == nil then
                            if not g_gui:getIsGuiVisible() then
                                g_gui:showYesNoDialog({
                                    title = string.format("%s - Version %s", mod.title, versionString),
                                    text = text,
                                    dialogType = DialogElement.TYPE_LOADING,
                                    callback = self.openModHubLink,
                                    target = nil,
                                    yesText = g_i18n:getText("button_ok"),
                                    noText = g_i18n:getText("button_modHubDownload")
                                })
                            end
                        else
                            print("\n" .. text .. "\n    - https://farming-simulator.com/mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0" .. "\n")
                            self.openModHubLink(true)
                        end
                    end
                end,

                openModHubLink = function(ignore)
                    if ignore == false then
                        local language = g_languageShort
                        local link = "mods.php?lang=en&country=be&title=fs2019&filter=org&org_id=129652&page=0"

                        if language == "de" or language == "fr" then
                            link = "mods.php?lang=" .. language .. "&country=be&title=fs2019&filter=org&org_id=129652&page=0"
                        end

                        openWebFile(link, "")
                    end

                    removeModEventListener(loadError)
                    loadError = nil
                end
            }

            addModEventListener(loadError)
        end
    end
end

init()
