--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2020

Author: GtX | Andy
Date: 20.12.2020

About:
Idea inspired by vehicle sales feature available in previous versions of FS/LS

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


StoreSalesManager = {}
StoreSalesManager.MAX_SALES = 4

local StoreSalesManager_mt = Class(StoreSalesManager)


function StoreSalesManager:new(isServer, isClient, customEnvironment, modDirectory, customMt)
    if g_storeSalesManager ~= nil then
        return nil
    end

    local self = setmetatable({}, customMt or StoreSalesManager_mt)

    self.isServer = isServer
    self.isClient = isClient

    self.customEnvironment = customEnvironment
    self.modDirectory = modDirectory

    self.debugActive = true

    self.firstTimeRun = true
    self.sales = {}

    self.brandSaleObjectsParts = modDirectory .. "shared/brandSaleObjects.i3d"
    self.brandSaleObjects = nil

    for i = 1, StoreSalesManager.MAX_SALES do
        local sale = StoreSale:new(isServer, isClient, customEnvironment, i == 1)

        if sale ~= nil then
            table.insert(self.sales, sale)
        end
    end

    if self.isServer then
        if g_currentMission.environment ~= nil then
            g_currentMission.environment:addHourChangeListener(self)
        end

        addConsoleCommand("ssReloadStoreSales", "Reload all store sales", "consoleCommandReloadAllSales", self)
        addConsoleCommand("ssStartStoreSale", "Starts a store sale on next hour change [brandSale]", "consoleCommandStartStoreSale", self)
    end

    addConsoleCommand("ssRenderStoreSales", "Render all store sales", "consoleCommandRenderStoreSales", self)

    return self
end

function StoreSalesManager:delete()
    if self.isServer then
        if g_currentMission.environment ~= nil then
            g_currentMission.environment:removeHourChangeListener(self)
        end

        removeConsoleCommand("ssReloadStoreSales")
        removeConsoleCommand("ssStartStoreSale")
    end

    removeConsoleCommand("ssRenderStoreSales")

    if self.salesRender ~= nil then
        g_currentMission:removeUpdateable(self.salesRender)
        self.salesRender:delete()
        self.salesRender = nil
    end

    if self.brandSaleObjects ~= nil then
        self.brandSaleObjects:delete()
        self.brandSaleObjects = nil
    end

    for _, sale in ipairs(self.sales) do
        sale:delete()
    end

    self.sales = {}
end

function StoreSalesManager:saveToXMLFile(xmlFile, key)
    local itemSalesIndex = 0
    local brandSalesIndex = 0

    for _, sale in ipairs(self.sales) do
        if sale:getIsBrandSale() then
            sale:saveToXMLFile(xmlFile, string.format("%s.brandSales.brandSale(%d)", key, brandSalesIndex))
            brandSalesIndex = brandSalesIndex + 1
        else
            sale:saveToXMLFile(xmlFile, string.format("%s.itemSales.itemSale(%d)", key, itemSalesIndex))
            itemSalesIndex = itemSalesIndex + 1
        end
    end
end

function StoreSalesManager:loadFromXMLFile(xmlFile, key)
    local itemSalesIndex = 0
    local brandSalesIndex = 0

    for _, sale in ipairs(self.sales) do
        if sale:getIsBrandSale() then
            sale:loadFromXMLFile(xmlFile, string.format("%s.brandSales.brandSale(%d)", key, brandSalesIndex))
            brandSalesIndex = brandSalesIndex + 1
        else
            sale:loadFromXMLFile(xmlFile, string.format("%s.itemSales.itemSale(%d)", key, itemSalesIndex))
            itemSalesIndex = itemSalesIndex + 1
        end
    end

    self.firstTimeRun = false
end

function StoreSalesManager:hourChanged()
    if self.isServer then
        local salesChanged = false

        local currentDay = g_currentMission.environment.currentDay
        local currentHour = g_currentMission.environment.currentHour

        for _, sale in ipairs(self.sales) do
            if sale:updateSale(currentDay, currentHour) then
                salesChanged = true
            end
        end

        if salesChanged then
            g_server:broadcastEvent(StoreSalesEvent:new(self.sales))
        end
    end
end

function StoreSalesManager:updateBrandSaleObjects(sale)
    if self.brandSaleObjects ~= nil then
        sale = sale or self.sales[1]

        if sale ~= nil and sale.isBrandSale then
            self.brandSaleObjects:updateObjects(sale.isActive, sale.discountPercent)
        end
    end
end

function StoreSalesManager:updateShopMenu()
    if g_gui:getIsGuiVisible() and (g_gui.currentGuiName == "ShopMenu" or g_gui.currentGuiName == "ShopConfigScreen") then
        local target = g_gui.currentGui.target

        if target ~= nil then
            if target.name == "ShopConfigScreen" then
                local screenElement = g_gui.screens[ShopMenu]

                if target.returnScreenName ~= "ShopMenu" or target.storeItem == nil or screenElement == nil or screenElement.target == nil then
                    return
                end

                -- Update the base price if in purchase screen, vehicle seems to be for configuration changes when owned
                target:updateData(target.storeItem, target.vehicle)
                target = screenElement.target
            end

            -- Only completely refresh if the 'SALES' brand is the one open to clear page of inactive sales
            if target.currentPage == target.pageShopItemDetails then
                if target.currentPage.categoryLabel ~= nil and target.currentPage.categoryLabel.text == "SALES" then
                    local brandIndex = g_brandManager:getBrandIndexByName("SALES")

                    if brandIndex ~= nil then
                        target.currentDisplayItems = target.shopController:getItemsByBrand(brandIndex) -- Set the variable so nothing breaks
                        target.currentPage:setDisplayItems(target.currentDisplayItems, false) -- Set the new display items, seems to clear without errors if there is none

                        return
                    end
                end
            end

            -- If it is not the 'SALES' brand then update all of the display items to remove icons and fix displayed pricing
            if target.updateCurrentDisplayItems ~= nil then
                target:updateCurrentDisplayItems()
            end
        end
    end
end

function StoreSalesManager:removeStoreSale(sale)
    return ListUtil.removeElementFromList(self.sales, sale)
end

function StoreSalesManager:getIsSaleItem(item)
    for _, sale in ipairs (self.sales) do
        if sale:getIsSaleItem(item) then
            return sale
        end
    end
end

function StoreSalesManager:getSaleById(id)
    local sale = self.sales[id]

    if sale == nil and id <= StoreSalesManager.MAX_SALES then
        sale = StoreSale:new(self.isServer, self.isClient, self.customEnvironment, id == 1)

        if sale ~= nil then
            self.sales[id] = sale
        end
    end

    return sale
end

function StoreSalesManager:getSales()
    return self.sales
end

function StoreSalesManager:consoleCommandReloadAllSales()
    if self.isServer then
        if #self.sales > 0 then
            local currentDay = g_currentMission.environment.currentDay
            local currentHour = g_currentMission.environment.currentHour

            for _, sale in ipairs (self.sales) do
                sale:setActive(false)
                sale:createSale(currentDay, currentHour)
            end

            g_server:broadcastEvent(StoreSalesEvent:new(self.sales))

            return "All shop sales reloaded"
        end

        return "Failed to reload shop sales"
    end

    return "Info: 'consoleCommandReloadAllSales' is a server only function"
end

function StoreSalesManager:consoleCommandStartStoreSale(isBrandSale)
    if self.isServer then
        isBrandSale = Utils.stringToBoolean(isBrandSale)

        local currentDay = g_currentMission.environment.currentDay
        local currentHour = g_currentMission.environment.currentHour

        for _, sale in ipairs (self.sales) do
            if sale.isBrandSale == isBrandSale then
                if not sale:getIsValid() then
                    sale:createSale(currentDay, currentHour)
                end

                if sale.isValid and not sale.isActive then
                    local startHour = currentHour + 1

                    if startHour == 24 then
                        startHour = 0
                    end

                    sale.startHour = startHour
                    sale.startDay = currentDay

                    if isBrandSale then
                        return "Store brand sale will start in the next hour"
                    else
                        return "Store item sale will start in the next hour"
                    end
                end
            end
        end

        return "Failed to start store sale, no slots available"
    end

    return "Info: 'consoleCommandStartStoreSale' is a server only function"
end

function StoreSalesManager:consoleCommandRenderStoreSales()
    if self.salesRender == nil then
        self.salesRender = StoreSalesRender:new(self.sales)
        g_currentMission:addUpdateable(self.salesRender)
    else
        g_currentMission:removeUpdateable(self.salesRender)
        self.salesRender:delete()
        self.salesRender = nil
    end

    return "Render store sales: " .. tostring(self.salesRender ~= nil)
end
