--[[
Interface: 1.7.1.0 b10490

Copyright (C) GtX (Andy), 2021

Author: GtX | Andy
Date: 15.03.2021

Contact:
https://forum.giants-software.com
https://github.com/GtX-Andy

Usage:
Loaded and used when a map includes the script callback 'modOnCreate.BrandSaleObjects' and allows brand sale signage and flags.
Script does not need to be added to map allowing for Console / PC maps to include support.

Important:
Not to be added to any mods / maps or modified from its current release form.
No changes are to be made to this script without permission from GtX | Andy

Darf nicht zu Mods / Maps hinzugefügt oder von der aktuellen Release-Form geändert werden.
An diesem Skript dürfen ohne Genehmigung von GtX | Andy keine Änderungen vorgenommen werden
]]


BrandSaleObjects = {}
local BrandSaleObjects_mt = Class(BrandSaleObjects)

function BrandSaleObjects.onCreate(nodeId)
    if g_storeSalesManager ~= nil and g_storeSalesManager.brandSaleObjects == nil then
        local brandSaleObjects = BrandSaleObjects:new(nodeId, g_storeSalesManager)

        if brandSaleObjects ~= nil and brandSaleObjects.isValid then
            g_storeSalesManager.brandSaleObjects = brandSaleObjects
        end
    end
end

function BrandSaleObjects:new(nodeId)
    local self = setmetatable({}, BrandSaleObjects_mt)

    self.nodeId = nodeId

    self.isValid = false
    self.isVisable = false

    self.objects = {}

    local numSharedI3DFiles = 0
    local filename = g_storeSalesManager.brandSaleObjectsParts

    if filename ~= nil and not fileExists(filename) then
        filename = nil
    end

    -- If this node contains children they will be hidden when a sale is active otherwise the given node is hidden instead
    local hideOnSaleNode = I3DUtil.indexToObject(nodeId, getUserAttribute(nodeId, "hideOnSaleNode"))

    for i = 0, getNumOfChildren(nodeId) - 1 do
        local node = getChildAt(nodeId, i)

        if node ~= hideOnSaleNode then
            local isSign = Utils.getNoNil(getUserAttribute(node, "isSign"), false)
            local objectType = isSign and "sign" or "flag"

            local useBase = not isSign and Utils.getNoNil(getUserAttribute(node, "useBase"), false) -- Flag only to display a base for concrete areas

            -- If attribute 'useSelf = true' then the node will be used and Store Sales items will not be used. Allowing for custom flags and signs
            if filename ~= nil and not Utils.getNoNil(getUserAttribute(node, "useSelf"), false) then
                local i3dNode = g_i3DManager:loadSharedI3DFile(filename, "", false, false, true)

                if i3dNode ~= 0 then
                    local childNode = getChild(i3dNode, objectType)

                    if childNode ~= nil then
                        link(node, childNode)
                        addToPhysics(childNode)
                    end

                    node = childNode
                    delete(i3dNode)

                    numSharedI3DFiles = numSharedI3DFiles + 1
                end
            end

            if node ~= nil then
                local object = {
                    node = node,
                    isSign = isSign
                }

                if isSign then
                    local display = I3DUtil.indexToObject(node, getUserAttribute(node, "displayNode"))

                    if display ~= nil then
                        object.display = display
                        I3DUtil.setNumberShaderByValue(display, 0, 0, true)
                    end
                else
                    if getNumOfChildren(node) > 0 then
                        local baseNode = getChildAt(node, 0)

                        if useBase then
                            setVisibility(baseNode, true)
                        else
                            delete(baseNode)
                        end
                    end
                end

                object.setCollision = Utils.getNoNil(getUserAttribute(node, "setCollision"), false)

                if object.setCollision then
                    object.collisionMask = getCollisionMask(node)
                    setCollisionMask(node,  0)
                end

                setVisibility(node, false)

                table.insert(self.objects, object)
            end
        end
    end

    if hideOnSaleNode ~= nil then
        local numChildren = getNumOfChildren(hideOnSaleNode)

        if numChildren > 0 then
            self.hideOnSaleObjects = {}

            if numChildren > 1 then
                for i = 0, getNumOfChildren(hideOnSaleNode) - 1 do
                    local node = getChildAt(nodeId, i)

                    table.insert(self.hideOnSaleObjects, {
                        node = node,
                        collisionMask = getCollisionMask(node),
                        setCollision = Utils.getNoNil(getUserAttribute(node, "setCollision"), false)
                    })
                end
            else
                self.hideOnSaleObjects[1] = {
                    node = hideOnSaleNode,
                    collisionMask = getCollisionMask(hideOnSaleNode),
                    setCollision = Utils.getNoNil(getUserAttribute(hideOnSaleNode, "setCollision"), false)
                }
            end
        end
    end

    for i = 1, numSharedI3DFiles do
        g_i3DManager:releaseSharedI3DFile(filename, "", true)
    end

    if #self.objects > 0 then
        self.isValid = true
        setVisibility(nodeId, true)
    end

    return self
end

function BrandSaleObjects:delete()
    self:updateObjects(false)
end

function BrandSaleObjects:updateObjects(isVisable, discontPercent)
    local isVisable = Utils.getNoNil(isVisable, false)

    if self.isVisable ~= isVisable then
        self.isVisable = isVisable

        if self.hideOnSaleObjects ~= nil then
            for _, object in ipairs (self.hideOnSaleObjects) do
                setVisibility(object.node, not isVisable)

                if object.setCollision then
                    setCollisionMask(object.node, not isVisable and object.collisionMask or 0)
                end
            end
        end

        for _, object in ipairs (self.objects) do
            if isVisable and object.display ~= nil then
                I3DUtil.setNumberShaderByValue(object.display, math.max(discontPercent or 0, 0), 0, true)
            end

            setVisibility(object.node, isVisable)

            if object.setCollision then
                setCollisionMask(object.node, isVisable and object.collisionMask or 0)
            end
        end
    end
end
